# model using example

library(DiagrammeR)
library(data.tree)
library(dplyr)
library(networkD3)


source("Functions.R")


D_G1 <- 8.0
D_S <- 8.2
D_G2 <- 8.5
D_M <- 9.0
a <- 0.124
b <- 0.115
k <- 3.35
C_min <- 0.018
alph <- 0.65
V_0 <- 1.0
Inht <- 4.0
VSData <- read.table("data/rates.dat", header = T) # VS-model output
VSData <- VSData[grepl("^1985", VSData$Data), ]
VSData <- subset(VSData, Day >= 144 & Day <= 281)


nrow(VSData) # season length
division <-
  CalculateCellsDivisionWithParents_VSControlled(
    D_G1, D_S, D_G2, D_M, a, b, k, alph, V_0, Inht, 5.0, C_min,
    GRates = VSData, s = 0.85
  )

division <- subset(division, TFromInitial <= nrow(VSData))




nrow(HowManyTerminalsAchieved(division, GrowthSeasonInDays = nrow(VSData))) # modelled production

# build division binary tree
pop <- as.Node(division)


f <- function(node) {
  if (is.null(node$InhtConc) == FALSE) {
    if (as.numeric(node$InhtConc) <= 0.018) {
      return("#a31008") # red leaves are cells, which gone to enlargement zone
    }
    else {
      (return("Black"))
    }
  }
  else {
    return("Black")
  }
}



# Tree design functions
GetEdgeLabel <- function(node) {
  label <-
    paste("cycle time = ", round(as.numeric(node$BirthGivingTime), digits = 3), sep = " ")
  
  
  return(label)
}


GetNodeLabel <- function(node) {
  if (is.null(node$InhtConc) == FALSE) {
    label <- paste("C =", round(as.numeric(node$InhtConc), digits = 2), "V =", round(as.numeric(node$V_init), digits = 2), sep = " ")
  }
  else {
    label <- "Dormant initial"
  }
  
  
  return(label)
}


#--
SetEdgeStyle(pop, fontname = "helvetica", label = GetEdgeLabel, color = f)
SetNodeStyle(pop, fontname = "helvetica", label = GetNodeLabel, color = f)
#--
SetGraphStyle(pop, rankdir = "LR")
plot(pop) # tree output
