
genVars <- c("a, b, k", "a, b, k and distribution coefficient",
             "only distribution coefficient")


ui <- fluidPage(


  # App title ----
  titlePanel("VS-CD model parameters"),

  # Sidebar layout with input and output definitions ----
  sidebarLayout(

    
    sidebarPanel(
      actionButton("resetDefault", "Start with test parameters"),

      h5("use this option to see  app visual facilities with default cell production data"),

      h2("    "),

      actionButton("reset", "Reset model parameters"),

      h2("  "),

      fileInput("file1", "Choose daily data file",
        multiple = FALSE,
        accept = c( # "text/csv",
          # "text/comma-separated-values,text/plain",
          ".dat"
        )
      ),


      h2("    "),
      numericInput("Year", "Input target year", 2002, min = 1000, max = 5000),
      numericInput("Sos", "Input start of the season day (DOY)", 133, min = 1, max = 366),
      numericInput("Eos", "Input end of the season day (DOY)", 263, min = 1, max = 366),
      # Input: Simple integer interval ----
      sliderInput("D_G1", "Size of dividing cell when it  passes from G1 to S:",
        min = 4, max = 15,
        value = 8.0, step = 0.1
      ),

      sliderInput("D_S", "Size of dividing cell when it  passes from S to G2:",
        min = 4, max = 15,
        value = 8.2, step = 0.1
      ),

      # Input: Decimal interval with step value ----
      sliderInput("D_G2", "Size of dividing cell when it  passes from G2 to M:",
        min = 4, max = 15,
        value = 8.5, step = 0.1
      ),

      sliderInput("D_M", "Size of dividing cell when it  begins to divide:",
        min = 4, max = 15,
        value = 9.0, step = 0.1
      ),

      sliderInput("a", "Parameter a:",
        min = 0, max = 1,
        value = 0.06, step = 0.001
      ),

      sliderInput("b", "Parameter b:",
        min = 0, max = 1,
        value = 0.11, step = 0.001
      ),

      sliderInput("k", "Parameter k:",
        min = 0, max = 5,
        value = 4.35, step = 0.001
      ),

      sliderInput("C_min", "Minimal concentration of inhibitors for division:",
        min = 0, max = 1,
        value = 0.018, step = 0.001
      ),


      sliderInput("alph", "Distribution coefficient:",
        min = 0, max = 1,
        value = 0.65, step = 0.01
      ),

      sliderInput("V_0", "Growth rate in S, G2 and M phases:",
        min = 0, max = 1,
        value = 1.0, step = 0.1
      ),

      sliderInput("Inht", "Inhibitor amount in initial cell:",
        min = 1, max = 10,
        value = 4.0, step = 0.1
      ),

      sliderInput("InitSize", "Size of initial cell:",
        min = 4, max = 10,
        value = 5.0, step = 0.1
      ),
      
      sliderInput("Stresscoef", "Stress coefficient:",
                  min = 0.1, max = 1.0,
                  value = 1.0, step = 0.1
      )

      #    sliderInput("SeasonLength", "Season length in days:",
      #               min = 100, max = 200,
      #              value = 100, step = 1)
    ),

    # Main panel for displaying outputs ----
    mainPanel(



      #   actionButton("reset", "Reset model parameters"),


      tabsetPanel(
        type = "tabs",
        tabPanel(
          "Tree View", h5("Tree of cell dividing iterations. First cell is initial, it divides multiple times. In a tree structure, child elements present maternal cells.
                                           Orange stages of cell fate are terminal, with no amount of ingibitor for next dividings.
                                           Cycle time is measured in days for each cell. C is for inhubitor amount, V is for growth speed for each cell."), grVizOutput("xx", width = "250%", height = "500%") # height = "500px")
        ),
        tabPanel(
          "Dinamic Cell View", h5("This is a model of camium radial range. Each square presents a mature cambial cell.
Left side is for initial cell, rigth side is for core wood.
Bright violet color is for new cell in radial range, lightblue color is for another sells in radial range.
                                                   "),

          
          uiOutput("diagram"),
          fluidRow(
            column(1, actionButton("prevButton", "Prev")),
            column(2, div(
              style = "width:40px;padding-left:5px;",
              fluidRow(verbatimTextOutput("nText"))
            )),
            column(3, actionButton("nextButton", "Next"))
          ),
          textOutput("CellInfo"), textOutput("CellInfo2")
        ),
        tabPanel(
          "Diagrams",
          h5("Each diagram describes distribution of
                              growth speed or inhibitor concentration during cambium ribbon surface.
                              Cell with 1 index is initial, cel with the biggest index is the closest cell to core wood. "),
          plotOutput("plot"), plotOutput("plot2"), plotOutput("plot3")
        )
      )
    )
  ),
  options(encoding = 'UTF-8')
)
