#

server <- function(input, output) {
  # reactive values ("global" variables for events)
  values <- reactiveValues(org = NULL, names = NULL, data = NULL, currentNumber = NULL)

  GetNodeColor <- function(node) # function depends on input value
  {
    if (is.null(node$InhtConc) == FALSE) {
      if (as.numeric(node$InhtConc) <= input$C_min) {
        return("#f09d05")
      }
      else {
        (return("Black"))
      }
    }
    else {
      return("Black")
    }
  }

  Fitness1 <- function(x) # function depends on input value
  {
    par_a <- x[1]
    par_b <- x[2]
    par_k <- x[3]
    cells <- CalculateCellsDivisionWithParents(input$D_G1, input$D_S, input$D_G2, input$D_M, par_a, par_b, par_k, input$alph, input$V_0, input$Inht, input$InitSize, input$C_min)
    cells <- subset(cells, TFromInitial <= (input$Eos - input$Sos + 1))
    produced <- HowManyTerminalsAchieved(cells)
    return(nrow(produced) * (-1))
  }


  # reactive events recalculate parameters
  modelRecalculate <- eventReactive(input$reset, {
    req(input$file1)

    VSData <- data.frame()

    tryCatch(
      {
        #application works only with output of VS-Model (look more at vs-genn web-site)
        VSData <- read.csv(input$file1$datapath,
          header = T,
          sep = "\t"
        )
        

       

        expr <- paste("^", input$Year, sep = "")

        VSData <- VSData[grepl(expr, VSData$Data), ]
        VSData <- subset(VSData, Day >= input$Sos & Day <= input$Eos)
      },
      error = function(e) {
        # return a safeError if a parsing error occurs
        stop(safeError(e))
      }
    )


    cells <- NULL
    if (nrow(VSData) > 0) {
      
      cells <- CalculateCellsDivisionWithParents_VSControlled(input$D_G1, input$D_S, input$D_G2, input$D_M, input$a, input$b, input$k, input$alph, input$V_0, input$Inht, input$InitSize, input$C_min, GRates = VSData, input$Stresscoef)
      cells <- subset(cells, TFromInitial <= nrow(VSData))
      values$data <- cells
    }
    else {
      # error
    }
  })

  nextButtonRecalculate <- eventReactive(input$nextButton, {
    number <- values$currentNumber
    ribbon <- BuildRibbonStructureFromTree(values$data, GrowthSeasonInDays = (input$Eos - input$Sos + 1))
    if ((number + 1) <= length(ribbon)) {
      values$currentNumber <- number + 1
    }
  })

  prevButtonRecalculate <- eventReactive(input$prevButton, {
    number <- values$currentNumber
    
    if ((number - 1) > 1) {
      values$currentNumber <- number - 1
    }
  })

  modelRecalculateByDefault <- eventReactive(input$resetDefault, {
    DataPath <- paste(here(), "/data/rates.dat", sep = "")
    VSData <- read.csv(DataPath,
      header = T,
      sep = "\t"
    )
    expr <- paste("^", input$Year, sep = "")

    VSData <- VSData[grepl(expr, VSData$Data), ]
    VSData <- subset(VSData, Day >= input$Sos & Day <= input$Eos)
    cells <- CalculateCellsDivisionWithParents_VSControlled(input$D_G1, input$D_S, input$D_G2, input$D_M, input$a, input$b, input$k, input$alph, input$V_0, input$Inht, input$InitSize, input$C_min, GRates = VSData, 0.8)
    cells <- subset(cells, TFromInitial <= nrow(VSData))
    values$data <- cells
  })


  # observe events for diagrams redraw

  observeEvent(input$reset, {
    modelRecalculate()

    cells <- NULL
    cells <- values$data



    # Tree builder
    population <- as.Node(cells)

    SetEdgeStyle(population, fontname = "helvetica", label = GetEdgeLabel)
    SetNodeStyle(population, fontname = "helvetica", label = GetNodeLabel, color = GetNodeColor)
    SetGraphStyle(population, rankdir = "LR")
    values$org <- population



    # re-generate chart
    output$xx <- renderGrViz({
      grViz(DiagrammeR::generate_dot(ToDiagrammeRGraph(values$org)), engine = "dot")
    })


    # rubbin visualization builder

    ribbon <- BuildRibbonStructureFromTree(cells, GrowthSeasonInDays = (input$Eos - input$Sos + 1))
    ribbonReactive <- ribbon

    r1 <- ribbon[[length(ribbon)]]


    values$currentNumber <- length(ribbon)

    output$nText <- renderText({
      return(length(ribbon))
    })


    sankey <- RedrawSankey(values$currentNumber, (input$Eos - input$Sos + 1), values$data)


    s <- sankey$data

    # print(s)

    links <- data.frame(
      source = s$ID,
      target = c(tail(s$ID, length(s$ID) - 1), " "),
      group = sankey$ribbonData$state,
      value = rep(1, nrow(s))
    )






    nodes <- data.frame(
      name_withInfo = c(paste("C = ", round(s$InhtConc, digits = 3), sep = ""), ""),
      name2 = c(
        as.character(links$source),
        as.character(links$target)
      ) %>% unique()
    )

    nodes$group <- as.factor(c("my_unique_group"))

    my_color <- 'd3.scaleOrdinal() .domain(["new", "old", "my_unique_group"]) .range(["#995aad", "#94d6e0", "#d3dbdb"])'

    links$IDsource <- match(links$source, nodes$name2) - 1
    links$IDtarget <- match(links$target, nodes$name2) - 1

    sn <- sankeyNetwork(
      Links = links, Nodes = nodes,
      Source = "IDsource", Target = "IDtarget",
      Value = "value", NodeID = "name_withInfo",
      colourScale = my_color,
      LinkGroup = "group", NodeGroup = "group",
      sinksRight = TRUE,
      nodeWidth = 20,
      nodePadding = 400,
      fontSize = 16
    )



    output$SankeyPlot <- renderSankeyNetwork({
      sn %>%
        onRender('function(el, x) {
                       d3.selectAll(".node text").attr("text-anchor", "begin").attr("x", 20);
                       }
                       ')
    })



    output$diagram <- renderUI({
      sankeyNetworkOutput("SankeyPlot", width = as.character(180 * values$currentNumber), height = as.character(180))
    })


    # CellInfo space builder

    newCell <- subset(links, group == "new")$source
    newCellInfo <- subset(s, ID == newCell)
    

    output$CellInfo <- renderText({
      return(paste("Growth season day is ", round(newCellInfo$TFromInitial, digits = 2), "\n", sep = ""))
    })
    output$CellInfo2 <- renderText({
      inhtInfo <- " "
      if (newCellInfo$InhtConc < input$C_min) {
        inhtInfo <- "Cell is terminated and go to enlargement stage"
      }
      else {
        inhtInfo <- "Cell is ready for new dividings"
      }

      return(paste("Newborn cell growth speed is ", round(newCellInfo$V_init, digits = 2),
        "\n",
        ", Inhibitor amount is ", round(newCellInfo$InhtAmount, digits = 4),
        ", Inhibitor concentration is ", round(newCellInfo$InhtConc, digits = 4), " (G2 to M stage)",

        ", cell spent ", round(newCellInfo$T_Cumulative, digits = 2), " days before diving. ",
        "\n",
        inhtInfo,
        "\n",
        sep = ""
      ))
    })

    CellPositionDiagrams <- list(
      function() {
        plot(s$V_init,
          type = "l", xlab = "cell position in radial file", lwd = 4, col = "lightblue",
          ylim = c(0, 1.15), ylab = " ",
          main = "Cell development kinetics depending on cell position in radial file"
          
        )
      },
      function() lines(s$InhtConc, type = "l", lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 0.2, y = 1.1, legend = c("cell growth rate, micron per day", "inhibitor concentration, rel.units"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
    )

    output$plot <- renderPlot({
      for (i in 1:3) CellPositionDiagrams[[i]]()
    })


    s1 <- cells[order(cells$TFromInitial), ] # show all division in a season

    CellInTimeDiagrams <- list(
      function() {
        plot(
          y = s1$V_init, x = s1$TFromInitial, type = "l", xlab = "day of the season",
          ylim = c(0, 1.15), ylab = " ",
          main = "Cell development kinetics depending on cell date of origin",
          lwd = 4, col = "lightblue", xlim = c(0, (input$Eos - input$Sos + 1) + 5)
        )
      },
      function() lines(y = s1$InhtConc, x = s1$TFromInitial, type = "l", lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 0.2, y = 1.1, legend = c("cell growth rate, micron per day", "inhibitor concentration, rel.units"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
    )

    output$plot2 <- renderPlot({
      for (i in 1:3) CellInTimeDiagrams[[i]]()
    })



    cellsToDayCounter <- c() # show ready-to-divide cells
    c <- data.frame(cells, stringsAsFactors = F)
    for (i in 1:(input$Eos - input$Sos + 1))
    {
      tmp <- subset(c, TFromInitial <= i & InhtConc >= input$C_min & Priority == 2)
      # print(i)
      # print(tmp)
      cellsToDayCounter <- c(cellsToDayCounter, as.numeric(nrow(tmp)))
    }


    cellsToDayCounter2 <- c() # show cells, which gone to enlargement zone
    c <- data.frame(cells, stringsAsFactors = F)
    for (i in 1:(input$Eos - input$Sos + 1))
    {
      tmp <- subset(c, TFromInitial <= i & InhtConc < input$C_min & Priority == 2)
      # print(i)
      # print(tmp)
      cellsToDayCounter2 <- c(cellsToDayCounter2, as.numeric(nrow(tmp)))
    }

    pfs <- list(
      function() {
        plot(
          y = cellsToDayCounter, x = 1:(input$Eos - input$Sos + 1), lwd = 4, col = "lightblue",
          type = "l", xlab = "day of the season", ylab = "cell amount",
          main = "Cell dividing trend in a season", xlim = c(0, (input$Eos - input$Sos + 1) + 5), 
          ylim = c(0, max(c(cellsToDayCounter2, cellsToDayCounter)))
        )
      },
      function() lines(y = cellsToDayCounter2, x = 1:(input$Eos - input$Sos + 1), lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 1, y = max(c(cellsToDayCounter2, cellsToDayCounter)) - 5, legend = c("Capable of dividing", "Uncapable of dividing"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
      # function() title(main='Fitted line plot')
    )

    output$plot3 <- renderPlot({
      for (i in 1:3) pfs[[i]]()
    })
    # end of observe event (input$reset) !
  })


  observeEvent(input$resetDefault, {
    modelRecalculateByDefault()
    cells <- NULL
    cells <- values$data
    # cells <- subset(cells, TFromInitial <= input$SeasonLength)


    # Tree builder
    population <- as.Node(cells)

    SetEdgeStyle(population, fontname = "helvetica", label = GetEdgeLabel)
    SetNodeStyle(population, fontname = "helvetica", label = GetNodeLabel, color = GetNodeColor)
    SetGraphStyle(population, rankdir = "LR")
    values$org <- population






    # re-generate chart
    output$xx <- renderGrViz({
      grViz(DiagrammeR::generate_dot(ToDiagrammeRGraph(values$org)), engine = "dot")
    })


    # rubbin visualization builder

    ribbon <- BuildRibbonStructureFromTree(cells, GrowthSeasonInDays = (input$Eos - input$Sos + 1))
    ribbonReactive <- ribbon

    r1 <- ribbon[[length(ribbon)]]


    values$currentNumber <- length(ribbon)

    output$nText <- renderText({
      return(length(ribbon))
    })


    sankey <- RedrawSankey(values$currentNumber, (input$Eos - input$Sos + 1), values$data)


    s <- sankey$data

    # print(s)

    links <- data.frame(
      source = s$ID,
      target = c(tail(s$ID, length(s$ID) - 1), " "),
      group = sankey$ribbonData$state,
      value = rep(1, nrow(s))
    )




    # links$group <- as.factor(c("my_unique_group"))

    #  print(links)

    nodes <- data.frame(
      name_withInfo = c(paste("C = ", round(s$InhtConc, digits = 3), sep = ""), ""),
      name2 = c(
        as.character(links$source),
        as.character(links$target)
      ) %>% unique()
    )

    nodes$group <- as.factor(c("my_unique_group"))

    my_color <- 'd3.scaleOrdinal() .domain(["new", "old", "my_unique_group"]) .range(["#995aad", "#94d6e0", "#d3dbdb"])'

    links$IDsource <- match(links$source, nodes$name2) - 1
    links$IDtarget <- match(links$target, nodes$name2) - 1

    sn <- sankeyNetwork(
      Links = links, Nodes = nodes,
      Source = "IDsource", Target = "IDtarget",
      Value = "value", NodeID = "name_withInfo",
      colourScale = my_color,
      LinkGroup = "group", NodeGroup = "group",
      sinksRight = TRUE,
      nodeWidth = 20,
      nodePadding = 400,
      fontSize = 16
    )



    output$SankeyPlot <- renderSankeyNetwork({
      sn %>%
        onRender('function(el, x) {
                       d3.selectAll(".node text").attr("text-anchor", "begin").attr("x", 20);
                       }
                       ')
    })



    output$diagram <- renderUI({
      # temp <- temp_dat()
      sankeyNetworkOutput("SankeyPlot", width = as.character(180 * values$currentNumber), height = as.character(180))
    })


    # CellInfo space builder

    newCell <- subset(links, group == "new")$source
    newCellInfo <- subset(s, ID == newCell)
    

    output$CellInfo <- renderText({
      return(paste("Growth season day is ", round(newCellInfo$TFromInitial, digits = 2), "\n", sep = ""))
    })
    output$CellInfo2 <- renderText({
      inhtInfo <- " "
      if (newCellInfo$InhtConc < input$C_min) {
        inhtInfo <- "Cell is terminated and go to enlargement stage"
      }
      else {
        inhtInfo <- "Cell is ready for new dividings"
      }

      return(paste("Newborn cell growth speed is ", round(newCellInfo$V_init, digits = 2),
        "\n",
        ", Inhibitor amount is ", round(newCellInfo$InhtAmount, digits = 4),
        ", Inhibitor concentration is ", round(newCellInfo$InhtConc, digits = 4), " (G2 to M stage)",

        ", cell spent ", round(newCellInfo$T_Cumulative, digits = 2), " days before diving. ",
        "\n",
        inhtInfo,
        "\n",
        sep = ""
      ))
    })

    CellPositionDiagrams <- list(
      function() {
        plot(s$V_init,
          type = "l", xlab = "cell position in radial file", lwd = 4, col = "lightblue",
          ylim = c(0, 1.15), ylab = " ",
          main = "Cell development kinetics depending on cell position in radial file"
        )
      },
      function() lines(s$InhtConc, type = "l", lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 0.2, y = 1.1, legend = c("cell growth rate, micron per day", "inhibitor concentration, rel.units"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
    )

    output$plot <- renderPlot({
      for (i in 1:3) CellPositionDiagrams[[i]]()
    })

   

    s1 <- cells[order(cells$TFromInitial), ] # show all divings

    CellInTimeDiagrams <- list(
      function() {
        plot(
          y = s1$V_init, x = s1$TFromInitial, type = "l", xlab = "day of the season",
          ylim = c(0, 1.15), ylab = " ",
          main = "Cell development kinetics depending on cell date of origin",
          lwd = 4, col = "lightblue", xlim = c(0, (input$Eos - input$Sos + 1) + 5)
        )
      },
      function() lines(y = s1$InhtConc, x = s1$TFromInitial, type = "l", lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 0.2, y = 1.1, legend = c("cell growth rate, micron per day", "inhibitor concentration, rel.units"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
    )

    output$plot2 <- renderPlot({
      for (i in 1:3) CellInTimeDiagrams[[i]]()
    })

  

    cellsToDayCounter <- c() 
    c <- data.frame(cells, stringsAsFactors = F)
    for (i in 1:(input$Eos - input$Sos + 1))
    {
      tmp <- subset(c, TFromInitial <= i & InhtConc >= input$C_min & Priority == 2)
      # print(i)
      # print(tmp)
      cellsToDayCounter <- c(cellsToDayCounter, as.numeric(nrow(tmp)))
    }


    cellsToDayCounter2 <- c() 
    c <- data.frame(cells, stringsAsFactors = F)
    for (i in 1:(input$Eos - input$Sos + 1))
    {
      tmp <- subset(c, TFromInitial <= i & InhtConc < input$C_min & Priority == 2)
      # print(i)
      # print(tmp)
      cellsToDayCounter2 <- c(cellsToDayCounter2, as.numeric(nrow(tmp)))
    }

    pfs <- list(
      function() {
        plot(
          y = cellsToDayCounter, x = 1:(input$Eos - input$Sos + 1), lwd = 4, col = "lightblue",
          type = "l", xlab = "day of the season", ylab = "cell amount",
          main = "Cell dividing trend in a season", xlim = c(0, (input$Eos - input$Sos + 1) + 5)
        )
      },
      function() lines(y = cellsToDayCounter2, x = 1:(input$Eos - input$Sos + 1), lwd = 4, col = "#995aad"),

      function() {
        legend(
          x = 1, y = max(c(cellsToDayCounter2, cellsToDayCounter)) - 5, legend = c("Capable of dividing", "Uncapable of dividing"),
          col = c("lightblue", "#995aad"), lty = 1:2, cex = 0.8
        )
      }
      # function() title(main='Fitted line plot')
    )

    output$plot3 <- renderPlot({
      for (i in 1:3) pfs[[i]]()
    })
  })

  observeEvent(input$prevButton, {
    prevButtonRecalculate()
    output$nText <- renderText({
      return(values$currentNumber)
    })

    sankey <- RedrawSankey(values$currentNumber, (input$Eos - input$Sos + 1), values$data)


    s <- sankey$data



    links <- data.frame(
      source = s$ID,
      target = c(tail(s$ID, length(s$ID) - 1), " "),
      group = sankey$ribbonData$state,
      value = rep(1, nrow(s))
    )



    nodes <- data.frame(
      name_withInfo = c(paste("C = ", round(s$InhtConc, digits = 3), sep = ""), ""),
      name2 = c(
        as.character(links$source),
        as.character(links$target)
      ) %>% unique()
    )

    nodes$group <- as.factor(c("my_unique_group"))

    my_color <- 'd3.scaleOrdinal() .domain(["new", "old", "my_unique_group"]) .range(["#995aad", "#94d6e0", "#d3dbdb"])'

    links$IDsource <- match(links$source, nodes$name2) - 1
    links$IDtarget <- match(links$target, nodes$name2) - 1

    sn <- sankeyNetwork(
      Links = links, Nodes = nodes,
      Source = "IDsource", Target = "IDtarget",
      Value = "value", NodeID = "name_withInfo",
      colourScale = my_color,
      LinkGroup = "group", NodeGroup = "group",
      sinksRight = TRUE,
      nodeWidth = 20,
      nodePadding = 400,
      fontSize = 16
    )



    output$SankeyPlot <- renderSankeyNetwork({
      sn %>%
        onRender('function(el, x) {
                       d3.selectAll(".node text").attr("text-anchor", "begin").attr("x", 20);
                       }
                       ')
    })



    output$diagram <- renderUI({
      sankeyNetworkOutput("SankeyPlot", width = as.character(180 * values$currentNumber), height = as.character(180))
    })


    # CellInfo space builder

    newCell <- subset(links, group == "new")$source
    newCellInfo <- subset(s, ID == newCell)
    print(newCell)
    print(s)

    output$CellInfo <- renderText({
      return(paste("Growth season day is ", round(newCellInfo$TFromInitial, digits = 2), "\n", sep = ""))
    })
    output$CellInfo2 <- renderText({
      inhtInfo <- " "
      if (newCellInfo$InhtConc < input$C_min) {
        inhtInfo <- "Cell is terminated and go to enlargement stage"
      }
      else {
        inhtInfo <- "Cell is ready for new dividings"
      }

      return(paste("Newborn cell growth speed is ", round(newCellInfo$V_init, digits = 2),
        "\n",
        ", Inhibitor amount is ", round(newCellInfo$InhtAmount, digits = 4),
        ", Inhibitor concentration C is ", round(newCellInfo$InhtConc, digits = 4), " (G2 to M stage)",

        ", cell spent ", round(newCellInfo$T_Cumulative, digits = 2), " days before diving. ",
        "\n",
        inhtInfo,
        "\n",
        sep = ""
      ))
    })
  })

  observeEvent(input$nextButton, {
    nextButtonRecalculate()
    output$nText <- renderText({
      return(values$currentNumber)
    })

    sankey <- RedrawSankey(values$currentNumber, (input$Eos - input$Sos + 1), values$data)

    s <- sankey$data



    links <- data.frame(
      source = s$ID,
      target = c(tail(s$ID, length(s$ID) - 1), " "),
      group = sankey$ribbonData$state,
      value = rep(1, nrow(s))
    )



    nodes <- data.frame(
      name_withInfo = c(paste("C = ", round(s$InhtConc, digits = 3), sep = ""), ""),
      name_byID = c(
        as.character(links$source),
        as.character(links$target)
      ) %>% unique()
    )

    nodes$group <- as.factor(c("my_unique_group"))

    my_color <- 'd3.scaleOrdinal() .domain(["new", "old", "my_unique_group"]) .range(["#995aad", "#94d6e0", "#d3dbdb"])'

    links$IDsource <- match(links$source, nodes$name_byID) - 1
    links$IDtarget <- match(links$target, nodes$name_byID) - 1

    sn <- sankeyNetwork(
      Links = links, Nodes = nodes,
      Source = "IDsource", Target = "IDtarget",
      Value = "value", NodeID = "name_withInfo",
      colourScale = my_color,
      LinkGroup = "group", NodeGroup = "group",
      sinksRight = TRUE,
      nodeWidth = 20,
      nodePadding = 400,
      fontSize = 16
    )



    output$SankeyPlot <- renderSankeyNetwork({
      sn %>%
        onRender('function(el, x) {
                              d3.selectAll(".node text").attr("text-anchor", "begin").attr("x", 20);
                 }
                              ')
    })



    output$diagram <- renderUI({
      sankeyNetworkOutput("SankeyPlot", width = as.character(180 * values$currentNumber), height = as.character(180))
    })



    # CellInfo space builder

    newCell <- subset(links, group == "new")$source
    newCellInfo <- subset(s, ID == newCell)
    print(newCell)
    print(s)

    output$CellInfo <- renderText({
      return(paste("Growth season day is ", round(newCellInfo$TFromInitial, digits = 2), "\n", sep = ""))
    })
    output$CellInfo2 <- renderText({
      inhtInfo <- " "
      if (newCellInfo$InhtConc < input$C_min) {
        inhtInfo <- "Cell is terminated and go to enlargement stage"
      }
      else {
        inhtInfo <- "Cell is ready for new dividings"
      }

      return(paste("Newborn cell growth speed is ", round(newCellInfo$V_init, digits = 2),
        "\n",
        ", Inhibitor amount is ", round(newCellInfo$InhtAmount, digits = 4),
        ", Inhibitor concentration C is ", round(newCellInfo$InhtConc, digits = 4), " (G2 to M stage)",

        ", cell spent ", round(newCellInfo$T_Cumulative, digits = 2), " days before diving. ",
        "\n",
        inhtInfo,
        "\n",
        sep = ""
      ))
    })




    # end of the event body!
  })
  observeEvent(input$startGeneticButton, {
    genVarsList <- c("a, b, k", "a, b, k and distribution coefficient", "only distribution coefficient")
    if (input$genVars == genVarsList[1]) {
      abk_alph_lower <- c(0, 0, 0)
      abk_alph_upper <- c(1, 1, 5)

      print(abk_alph_lower)


      res <- DEoptim(Fitness1, lower = abk_alph_lower, upper = abk_alph_upper, control = DEoptim.control(itermax = 2, NP = 10))
     
    }
    else if (input$genVars == genVarsList[2]) {

    }
    else if (input$genVars == genVarsList[3]) {

    }
  })


  # data to redraw Sankey plot function
  RedrawSankey <- function(actualDividingStep, GrowthSeasonLength, cellsDivision) {
    division <- cellsDivision
    GrowthSeasonInDays <- GrowthSeasonLength

    ribbon <- BuildRibbonStructureFromTree(division, GrowthSeasonInDays)

    ribbonData <- BuildRibbonStructureForAnalasys(ribbon, division, customIndex = actualDividingStep)

    dataToCurves <- GetDataForRibbonStructure(ribbonData$cellDivisionActualChanges, division)

    tmp_list <- list()

    tmp_list$ribbonData <- ribbonData
    tmp_list$data <- dataToCurves

    return(tmp_list)
  }


  # end of server function
}






# Tree design functions
GetEdgeLabel <- function(node) {
  label <-
    paste("cycle time = ", round(as.numeric(node$BirthGivingTime), digits = 3), sep = " ")


  return(label)
}


GetNodeLabel <- function(node) {
  if (is.null(node$InhtConc) == FALSE) {
    label <- paste("C =", round(as.numeric(node$InhtConc), digits = 2), "V =", round(as.numeric(node$V_init), digits = 2), sep = " ")
  }
  else {
    label <- "Dormant initial"
  }


  return(label)
}
